import os
import json
import configparser


def mkdir_p(path):
    if not os.path.isdir(path):
        os.makedirs(path, exist_ok=True)


def cp(content, path, overwrite=True):
    if overwrite:
        with open(path, 'wb') as fp:
            fp.write(content)


def read_json(path):
    with open(path, encoding='utf-8') as fp:
        return json.loads(fp.read())


def read_conf(path, header=None):
    conf = configparser.ConfigParser()
    conf.read(path)
    if header:
        return conf[header]
    return conf


def parse_jsonp(jsonp, **kwargs):
    js = jsonp[jsonp.index("(") + 1: jsonp.rindex(")")]
    return json.loads(js, **kwargs)


def get_file_extension(path):
    fn, ext = os.path.splitext(path)
    return ext



import pandas as pd


from scrapgo.utils.datecuts import slice_dates
from .utils import starts_after, connect_db, get_today_str_date, print_db_progress

from .scraper import *
from .settings import MIN_START_DATE, MAX_END_DATE
from .db.vars import (
    펀드정보테이블명, 지수테이블명,
    결산테이블명, 결산테이블컬럼매핑, 

    표준코드, 펀드명, 회사코드, 설정일, 스크랩여부, 상환여부, # FUNDINFO_TABLE
    기준일자, # FUNDINDEX_TABLE
    회계기말, 구분 # FUNDSETTTLE_TABLE
)


@starts_after(table_name=펀드정보테이블명, date_column_name=설정일)
@slice_dates(by='year')
def get_kofia_fundlist(**kwargs):
    sc = KofiaFundListScraper()
    r = sc.scrap(**kwargs)
    return pd.DataFrame(r['fund_list'])


def get_kofia_fund_detail(fund_std_code, **kwargs):
    kfinfo = KofiaFundInfoScraper()
    r = kfinfo.scrap(fund_std_code=fund_std_code)
    df_detail = pd.DataFrame(r['fund_detail'])
    df_etc = pd.DataFrame(r['fund_etc'])
    df_detail = pd.merge(df_detail, df_etc)
    return df_detail


@starts_after(table_name=펀드정보테이블명, date_column_name=설정일)
@slice_dates(by='month')
def get_kofia_fund_detail_list(**kwargs):
    sc = KofiaFundListScraper()
    r = sc.scrap(**kwargs)
    fund_list =  pd.DataFrame(r['fund_list'])
    details, etcs = [], []
    for index, fund in fund_list.iterrows():
        fund_std_code = fund['표준코드']
        scd = KofiaFundInfoScraper()
        r = scd.scrap(fund_std_code=fund_std_code)
        detail = pd.DataFrame(r['fund_detail'])
        etc = pd.DataFrame(r['fund_etc'])
        df = pd.merge(detail, etc, on='표준코드')
        details.append(df)
    detail = pd.concat(details, ignore_index=True)
    return pd.merge(fund_list, detail, on='표준코드')


@connect_db
def get_kofia_price_progress(db=None, **kwargs):
    fund_list = db.select(
        펀드정보테이블명, 
        columns=[표준코드, 회사코드, 설정일, 펀드명],
        where=f'{스크랩여부}="Y" AND {상환여부}="N"'
    )

    total_count = fund_list.shape[0]
    for i, (fund_std_code, company_code, start_date, fund_name) in enumerate(fund_list[[표준코드, 회사코드, 설정일, 펀드명]].values):
        initial_date = start_date
        if db:
            date = db.max(
                지수테이블명, 기준일자,
                where=f'{표준코드}="{fund_std_code}"'
            )
            if date:
                if date == get_today_str_date():
                    print(f"이미 {fund_std_code}({fund_name}) 의 오늘 자 지수 정보가 존재합니다.")
                    continue
                start_date = date
        sc = KofiaPriceProgressScraper()
        r = sc.scrap(
            fund_std_code=fund_std_code, company_code=company_code,
            start_date=start_date, end_date=MAX_END_DATE
        )
        df = pd.DataFrame(r['price_progress'])
        prefix = f"{fund_name}(설정일:{initial_date})에 대한 {df.shape[0]} 개의 새로운 지수 정보 저장중..."
        print_db_progress(total_count, i, 1, prefix=prefix)
        yield df


@starts_after(table_name=결산테이블명, date_column_name=회계기말)
@slice_dates(by='month')
def get_kofia_settle_exso_list(apply=펀드정보테이블명, **kwargs):
    sc = KofiaSettleExSoByDateScraper()
    r = sc.scrap(**kwargs)
    df =  pd.DataFrame(r['fund_exso_by_date'])
    return df





# import os
import pandas as pd

from scrapgo.utils.fileutils import get_file_extension

EXCEL_FILE_EXTENSIONS = ['.xlsx', '.xls', 'xls', 'xlsx', 'excel']
CSV_FILE_EXTENSIONS = ['.csv', 'csv']


def path2dataframe(path, **kwargs):
    ext = get_file_extension(path)
    if ext in EXCEL_FILE_EXTENSIONS:
        dataframe = pd.read_excel(path)
    elif ext in CSV_FILE_EXTENSIONS:
        dataframe = pd.read_csv(path)
    else:
        raise ValueError(f"The File Extension {ext} is not Supported!")
    return dataframe


def dataframe2path(dataframe, filename, extension='csv', index=False, **kwargs):
    if extension in CSV_FILE_EXTENSIONS:
        path = filename + '.csv'
        dataframe.to_csv(path, index=index, **kwargs)
    elif extension in EXCEL_FILE_EXTENSIONS:
        path = filename + '.xlsx'
        dataframe.to_excel(path, index=index, **kwargs)
    elif extension is None:
        print(dataframe.head())
    else:
        msg = f"Output file extension must in xlsx, csv"
        raise ValueError(msg)



FUND_LIST_COLUMNS = {
    'tmpv1': '회사',
    'tmpv2': '펀드명',
    'tmpv3': '설정일',
    'tmpv4': '펀드유형',
    'tmpv5': '투자지역',
    'tmpv8': '수탁회사',
    'tmpv9': '대표판매사',
    'tmpv10': '표준코드'
}

FUND_DETAIL_COLUMNS = {
    'vfundgbnm': '구분',
    'ufundtypnm': '상품분류',
    'vadditionalestmtdnm': '추가/단위구분',
    'establishmentdt': '설정일',
    'classcd': '분류코드',
    'shortcd': '단축코드',
    'trustacctrm': '신탁회계기간',
    'vinvestrgngbnm': '투자지역구분',
    'vsalergngbnm': '판매지역구분',
    'vprofittypecdnm': '운용실적공시분류',
    'vtraitdivnm': '특성분류',
    'vpripubgbnm': '공시/사모구분',
    'establishmentcot': '최소설정기준가격',
    'trusttrm': '신탁기간',
    'managerewrate': '운용보수',
    'salerewrate': '판매보수',
    'trustrewrate': '수탁보수',
    'generalofctrtrewrate': '일반사무수탁보수',
    'rewsum': '보수합계',
    'ter': '총비율이율(TER)',
    'frontendcmsrate': '선취수수료',
    'backendcmsrate': '후취수수료',
    'vmanagecompnm': '운용회사',
    'vgeneralofctrtcompnm': '일반사무수탁회사',
    'vtrustcompnm': '수탁회사',
    'companycd': '회사코드',
    # 추가 정보
    'uoriginalamt': '설정원본',
    'netasstotamt': '순자산총액',

}

PRICE_PROGRESS_COLUMNS = {
    'standarddt': '기준일자',
    'standardcot': '기준가격',
    'vbefdayfltstdcot': '전일대비등락',
    'standardassstdcot': '과표기준가격',
    'uoriginalamt': '설정원본',
    'kospiepn': 'KOSPI',
    'kospi200epn': 'KOSPI200',
    'kosdaqepn': 'KOSDAQ',
    'tbondbnd3y': '국공채(3년만기)',
    'companybnd3y': '회사채(3년만기)',
}

SETTLE_EXSO_COLUMNS = {
    'trustaccsrt': '신탁회계기초',
    'trustaccend': '신탁회계기말',
    'vpassdaycnt': '경과일수',
    'standardcot': '기준',
    'standardasscot': '과표',
    'uoriginalamt': '설정원본',
    'vsettlegbnm': '결산'
}



import time
import functools
from collections import abc


def retry(func):
    @functools.wraps(func)
    def wrapper(self, url, **kwargs):
        if isinstance(self.RETRY_INTERVAL_SECONDS, abc.Iterable):
            for i, sec in enumerate(self.RETRY_INTERVAL_SECONDS):
                try:
                    r = func(self, url, **kwargs)
                except Exception as e:
                    print('exception', e)
                    err = f"Request {url} Failed, retry after {sec}sec(trys: {i+1})"
                    print(err)
                    time.sleep(sec)
                else:
                    return r
            else:
                raise Exception(f'Request {url} Failed!')
        else:
            r = func(self, *args, **kwargs)
        return r
    return wrapper



import functools
from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta
from dateutil.parser import parse
from dateutil.rrule import rrule, MONTHLY, YEARLY

from .settings import OUTPUT_FILE_FORMAT, DEFAULT_AGO_DAYS, DATESTR_FMT, MIN_START_DATE
from scrapgo.lib.dataframe import TableFrame

def datetime2str(datetime, fmt=DATESTR_FMT, **kwargs):
    return datetime.strftime(fmt)


def get_today_str_date(fmt=DATESTR_FMT, **kwargs):
    today = datetime.today()
    str_today = today.strftime(fmt)
    return str_today


def get_ago_str_date(fmt=DATESTR_FMT, **kwargs):
    today = datetime.today()
    ago = today - relativedelta(**kwargs)
    str_ago = ago.strftime(fmt)
    return str_ago


def get_date_ago_range(start_date=None, end_date=None, days_ago=None, months_ago=None, years_ago=None, default_ago_days=7, **kwargs):
    end_date = end_date or get_today_str_date()

    if start_date:
        start_date = start_date
    elif days_ago:
        start_date = get_ago_str_date(days=days_ago)
    elif months_ago:
        start_date = get_ago_str_date(months=months_ago)
    elif years_ago:
        start_date = get_ago_str_date(years=years_ago)
    else:
        start_date = get_ago_str_date(days=default_ago_days)

    if start_date > end_date:
        raise ValueError('start_date > end_date')
    return start_date, end_date



def parse_date(date, fmt=None, **kwargs):
    if isinstance(date, datetime):
        d = date
    elif isinstance(date, str):
        d = parse(date)
    if fmt is not None:
        return d.strftime(fmt)
    return d


def get_first_date(date, of='year', fmt=None, **kwargs):
    d = parse_date(date)
    if of == 'year':
        y, m, d = d.year, 1, 1
    else:
        y, m, d = d.year, d.month, 1
    fdate = datetime(y, m, d)
    return parse_date(fdate, fmt=fmt)


def get_last_date(date, of='year', fmt=None, **kwargs):
    d = parse_date(date)
    if of == 'year':
        ndate = d + relativedelta(years=1)
    else:
        ndate = d + relativedelta(months=1)
    nfdate = get_first_date(ndate, of=of)
    ldate = nfdate - relativedelta(days=1)
    return parse_date(ldate, fmt=fmt)



def starts_after(table_name=None, date_column_name=None, where=None):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, start_date=None, end_date=None, output=None, **kwargs):
            if start_date is None:
                if output in OUTPUT_FILE_FORMAT:
                    start_date = get_ago_str_date(days=DEFAULT_AGO_DAYS)
                elif output in ['db'] and all((table_name, date_column_name)):
                    db = TableFrame(**kwargs)
                    start_date = db.max(table_name, date_column_name, where) or MIN_START_DATE
                else:
                    start_date = get_ago_str_date(days=DEFAULT_AGO_DAYS)
            if end_date is None:
                end_date = get_today_str_date()
            r = func(*args, start_date=start_date, end_date=end_date, output=output, **kwargs)
            return r
        return wrapper
    return decorator

def connect_db(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        db = TableFrame(**kwargs)
        return func(*args, db=db, **kwargs)
    return wrapper


def print_db_progress(total_count, index, print_on=1000, prefix="", postfix=""):
    if index % print_on == 0:
        now = datetime.now()
        pct = round((index*100)/total_count, 2)
        log = f"{prefix}{index}/{total_count}({pct}%) {now.strftime('%Y%m%d %H:%M:%S')}{postfix}"        
        print(log)



from types import GeneratorType
from collections import namedtuple, abc

import pandas as pd

from scrapgo.lib.dataframe import dataframe2path, TableFrame

from .settings import OUTPUT_FILE_FORMAT

from .apps import get_kofia_fundlist, get_kofia_fund_detail_list, get_kofia_price_progress, get_kofia_settle_exso_list
from .db.vars import (
    구분, 표준코드, 상환여부,

    펀드정보테이블명, 펀드정보테이블기준키, 펀드정보테이블컬럼매핑,
    지수테이블명, 지수테이블기준키, 지수테이블컬럼매핑,
    결산테이블명, 결산테이블기준키, 결산테이블컬럼매핑,
)


App = namedtuple('App', 'prefix table uniques mapping updateto', defaults=('Fund', None, None, None, None,))

apps = {
    get_kofia_fundlist: App(
        'FundList_{start_date}~{end_date}',
    ),
    get_kofia_fund_detail_list: App(
        'FundDetailList_{start_date}~{end_date}',
        펀드정보테이블명, 펀드정보테이블기준키, 펀드정보테이블컬럼매핑
    ),
    get_kofia_price_progress: App(
        'FundPriceProgress',
        지수테이블명, 지수테이블기준키, 지수테이블컬럼매핑
    ),
    get_kofia_settle_exso_list: App(
        'FundSettleExsoList_{start_date}~{end_date}',
        결산테이블명, 결산테이블기준키, 결산테이블컬럼매핑,
        펀드정보테이블명 # 정보 받은 후 바로 업데이트 시도
    )
}

def pipe(app, **kwargs):
    output = kwargs.get('output')
    dbconn = kwargs.get('db_conf_path')
    meta = apps.get(app)
    if meta is None:
        raise ValueError(
            f"{app.__name__}은 설정 된 출력 파이프가 없습니다. pipeline.py 의 apps에서 지정할 수 있습니다."
        )
    df = app(**kwargs)
    if not isinstance(df, GeneratorType):
        df = [df]
    if output in OUTPUT_FILE_FORMAT or output is None:
        dataframe = pd.concat(df, ignore_index=True)
        if output is None:
            print(dataframe.head())
        else:
            filename = meta.prefix.format(**kwargs)
            dataframe2path(dataframe, filename, extension=output)
    elif output in ['db']:
        db = TableFrame(
            db_conf_path=dbconn
        )
        if not meta.table:
            raise ValueError("DB 로 Insert 할 테이블명이 설정 되지않았 습니다.")
        for dataframe in df:
            db.insert(
                dataframe, meta.table, meta.uniques, meta.mapping
            )
            if meta.updateto:
                table_for_updated = meta.updateto
                db.update(
                    dataframe,
                    table_for_updated,
                    index='표준코드',
                    pk=표준코드,
                    source_column='구분',
                    dest_column=상환여부,
                    valuemap={'상환': 'Y'},
                    default='N',
                )
    else:
        print(df.head())




import functools
from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta
from dateutil.parser import parse
from dateutil.rrule import rrule, MONTHLY, YEARLY


DATESTR_FMT = "%Y%m%d"


def parse_date(date, fmt=None):
    if isinstance(date, datetime):
        d = date
    elif isinstance(date, str):
        d = parse(date)
    if fmt is not None:
        return d.strftime(fmt)
    return d


def get_first_date(date, of='year', fmt=None):
    d = parse_date(date)
    if of == 'year':
        y, m, d = d.year, 1, 1
    else:
        y, m, d = d.year, d.month, 1
    fdate = datetime(y, m, d)
    return parse_date(fdate, fmt=fmt)


def get_last_date(date, of='year', fmt=None):
    d = parse_date(date)
    if of == 'year':
        ndate = d + relativedelta(years=1)
    else:
        ndate = d + relativedelta(months=1)
    nfdate = get_first_date(ndate, of=of)
    ldate = nfdate - relativedelta(days=1)
    return parse_date(ldate, fmt=fmt)

 

def slice_date_range(start_date, end_date, by='year', fmt=DATESTR_FMT, **kwargs):
    start_date = parse_date(start_date)
    end_date = parse_date(end_date)
    ini_start_date = get_first_date(start_date, of=by)
    fin_end_date = get_last_date(end_date, of=by)
    freq = YEARLY if by == 'year' else MONTHLY
    date_list = list(rrule(freq=freq, dtstart=ini_start_date,
                           until=fin_end_date, interval=1))
    if start_date not in date_list:
        date_list.insert(0, start_date)
    for i, d in enumerate(date_list):
        if d < start_date:
            continue
        s = d
        e = get_last_date(s, of=by)
        if e >= end_date:
            e = end_date
        yield parse_date(s, fmt), parse_date(e, fmt)

def slice_dates(by='month'):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, start_date=None, end_date=None, **kwargs):
            for s, e in slice_date_range(start_date, end_date, by=by, fmt=DATESTR_FMT, **kwargs):
                r = func(*args, start_date=s, end_date=e, **kwargs)
                yield r
        return wrapper
    return decorator



import os
import re
import json
import math
from pprint import pprint
from urllib.parse import unquote
from datetime import datetime, timedelta

from scrapgo import LinkRelayScraper, urlpattern, url, root

from .payloaders import (
    get_fund_list_payload,
    get_fund_detail_payload,
    get_fund_etc_payload,
    get_price_change_progress_payload,
    get_fund_exso_payload,
    get_fund_exso_payload_by_date,
)
from .scrap_mappings import (
    SCRAPMAP_FUND_LIST_COLUMNS,
    SCRAPMAP_FUND_DETAIL_COLUMNS,
    SCRAPMAP_PRICE_PROGRESS_COLUMNS,
    SCRAPMAP_SETTLE_EXSO_COLUMNS,
    SCRAPMAP_SETTLE_EXSO_BY_DATE_COLUMNS,
)


class KofiaScraper(LinkRelayScraper):
    CACHE_NAME = 'PROFP_SCRAP_CACHE'
    REQUEST_DELAY = 0
    RETRY_INTERVAL_SECONDS = 10, 100, 1000,
    # REQUEST_LOGGING =


class KofiaFundListScraper(KofiaScraper):
    CACHE_NAME = 'PROFP_KOFIA_FUNDLIST_CACHE'
    LINK_RELAY = [
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='fund_list_payloader',
            parser='fund_list_parser',
            name='fund_list'
        )
    ]

    def get_request_log(self, *args, **kwargs):
        # print('get_request_log:', kwargs)
        log = super().get_request_log(*args, **kwargs)
        return "FUND_LIST: {log} {start_date}~{end_date}".format(log=log, **kwargs)

    def fund_list_payloader(self, **kwargs):
        # log = f"Retrieve FundList by Date Range: {start_date}~{end_date}"
        # print(log)
        payload = get_fund_list_payload(**kwargs)
        yield payload

    def fund_list_parser(self, response, **kwargs):
        soup = response.scrap.soup
        fund_list = self.parse_xml_table_tag(
            soup, 'selectmeta', SCRAPMAP_FUND_LIST_COLUMNS)
        return fund_list


class KofiaFundInfoScraper(KofiaScraper):
    # REQUEST_LOGGING = False
    LINK_RELAY = [
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='fund_detail_payloader',
            parser='fund_detail_parser',
            name='fund_detail'
        ),
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='fund_etc_payloader',
            parser='fund_etc_parser',
            name='fund_etc',
        ),
    ]

    def get_request_log(self, *args, **kwargs):
        log = super().get_request_log(*args, **kwargs)
        return "FUND_DETAIL: {log} {fund_std_code}".format(log=log, **kwargs)

    def fund_detail_payloader(self, fund_std_code):
        payload = get_fund_detail_payload(fund_std_code)
        # self.fund_std_code = fund_std_code
        yield payload

    def fund_detail_parser(self, response, fund_std_code, **kwargs):
        soup = response.scrap.soup
        fund = self.parse_xml_table_tag(
            soup, 'comfundbasinfooutdto', SCRAPMAP_FUND_DETAIL_COLUMNS,
            many=False
        )
        fund['표준코드'] = fund_std_code
        self.company_code = fund['회사코드']
        return fund

    def fund_etc_payloader(self, fund_std_code):
        payload = get_fund_etc_payload(fund_std_code, self.company_code)
        yield payload

    def fund_etc_parser(self, response, fund_std_code):
        soup = response.scrap.soup
        etc = self.parse_xml_table_tag(
            soup, 'comfundstdcotinfodto', SCRAPMAP_FUND_DETAIL_COLUMNS,
            many=False
        )
        etc['표준코드'] = fund_std_code
        # etc['회사코드'] = self.company_code
        return etc


class KofiaPriceProgressScraper(KofiaScraper):
    CACHE_NAME = 'PROFP_KOFIA_PRICE_PROGRESS_CACHE'
    LINK_RELAY = [
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='price_progress_payloader',
            parser='price_progress_parser',
            refresh=True,
            name='price_progress',
        )
    ]

    def get_request_log(self, *args, **kwargs):
        log = super().get_request_log(*args, **kwargs)
        return "PriceProgress: {log} {fund_std_code} {start_date}~{end_date}".format(log=log, **kwargs)

    def price_progress_payloader(self, **kwargs):
        payload = get_price_change_progress_payload(**kwargs)
        yield payload

    def price_progress_parser(self, response, fund_std_code, company_code, **kwargs):
        soup = response.scrap.soup
        price_progresses = self.parse_xml_table_tag(
            soup, 'pricemodlist', SCRAPMAP_PRICE_PROGRESS_COLUMNS
        )
        for progress in price_progresses:
            progress['표준코드'] = fund_std_code
            progress['fundinfo_id'] = fund_std_code
        return price_progresses


class KofiaSettleExSoScraper(KofiaScraper):
    CACHE_NAME = 'PROFP_KOFIA_SETTLE_EXSO_CACHE'
    LINK_RELAY = [
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='fund_exso_payloader',
            parser='fund_exso_parser',
            refresh=True,
            name='fund_exso',
        ),
    ]

    def fund_exso_payloader(self, fund_std_code, company_code, **kwargs):
        payload = get_fund_exso_payload(fund_std_code, company_code)
        return payload

    def fund_exso_parser(self, response, fund_std_code, **kwargs):
        soup = response.scrap.soup
        exsos = self.parse_xml_table_tag(
            soup, 'settleexlist', SCRAPMAP_SETTLE_EXSO_COLUMNS)
        for exso in exsos:
            exso['표준코드'] = fund_std_code
        return exsos


class KofiaSettleExSoByDateScraper(KofiaScraper):
    CACHE_NAME = 'PROFP_KOFIA_SETTLE_EXSO_CACHE'
    LINK_RELAY = [
        url(
            'http://dis.kofia.or.kr/proframeWeb/XMLSERVICES/',
            payloader='fund_exso_by_date_payloader',
            parser='fund_exso_by_date_parser',
            name='fund_exso_by_date',
        ),
    ]

    def fund_exso_by_date_payloader(self, start_date, end_date, **kwargs):
        payload = get_fund_exso_payload_by_date(start_date, end_date)
        log = f"Retrieve Fund Exso by Date Range: {start_date}~{end_date}"
        print(log)
        return payload

    def fund_exso_by_date_parser(self, response, **kwargs):
        soup = response.scrap.soup
        exsos = self.parse_xml_table_tag(
            soup, 'selectmeta',
            many=True,
            column_mapping=SCRAPMAP_SETTLE_EXSO_BY_DATE_COLUMNS
        )
        return exsos



from collections import OrderedDict


def parse_record(soup, tag, column_mapping, many=True):
    records = [
        OrderedDict(
            (column_mapping.get(r.name, r.name), r.text)
            for r in r.children
            if r.name in column_mapping
        )
        for r in soup(tag)
    ]
    if many is True:
        return records
    if records:
        return records[0]
    return {}



import argparse
import os
import sys

import pandas as pd

from practice.naver_kin_scraper.kin import naver_kin_with_image
from practice.naver_webtoon_scraper.webtoon2 import retrive_webtoon
from practice.durginfo.druginfo_scraper import drug_search
from practice.kofia_scraper import (
    pipe,
    get_kofia_fundlist, get_kofia_fund_detail_list, get_kofia_price_progress, get_kofia_settle_exso_list
)

sys.path.append('.')


MEDIA_ROOT = 'media'


def main():
    argparser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter, description="TEST Scraper"
    )
    argparser.add_argument('keywords', help='검색할 키워드들 나열', nargs='*')
    argparser.add_argument('-titleId', '--titleId')

    argparser.add_argument('-search', '--search', nargs='?')

    argparser.add_argument('-start', '--start', type=int, default=0)
    argparser.add_argument('-end', '--end', type=int, default=0)
    argparser.add_argument('-sd', '--start_date', type=str)
    argparser.add_argument('-ed', '--end_date', type=str)
    argparser.add_argument('-o', '--output', type=str)
    argparser.add_argument('-fund_std_code', '--fund_std_code')
    argparser.add_argument('-cd', '--code')
    argparser.add_argument('-conn', '--db_conf_path', type=str)

    args = argparser.parse_args()
    try:
        app = args.keywords[0]
    except:
        return

    context = {
        'save_to': MEDIA_ROOT
    }

    if app in ['webtoon', 'toon']:
        context['titleId'] = args.titleId
        retrive_webtoon(context)

    if app in ['kin']:
        context['save_to'] = os.path.join(MEDIA_ROOT, 'kin')
        naver_kin_with_image(params={'query': args.search}, context=context)

    if app in ['druginfo', 'di']:
        params = {
            'q': args.search
        }
        drug_search(params)

    kwargs = dict(args._get_kwargs())
    if app in ['kofia']:
        sub_app = args.keywords[1]
        if sub_app in ['ls']:
            apply = get_kofia_fundlist
        elif sub_app in ['ls-al']:
            apply = get_kofia_fund_detail_list
        elif sub_app in ['pg']:
            apply = get_kofia_price_progress
        elif sub_app in ['ex']:
            apply = get_kofia_settle_exso_list

        else:
            raise ValueError(f"{sub_app} 은 명령어 리스트에 존재하지 않습니다.")
        pipe(apply, **kwargs)
        # print(df.head())


if __name__ == "__main__":
    main()



import pandas as pd


def _get_index_mask(dataframe, other, index, how):
    df_index = dataframe[index].set_index(index).index
    oth_index = other[index].set_index(index).index
    mask = df_index.isin(oth_index)
    if how == 'diff':
        return ~mask
    return mask


def _filter_by_index_mask(dataframe, index, mask):
    dataframe = dataframe.set_index(index)
    filterd = dataframe.loc[mask]
    dataframe = filterd.reset_index()
    return dataframe


def get_intersect_with(dataframe, other, index):
    mask = _get_index_mask(dataframe, other, index, 'intersect')
    dataframe = _filter_by_index_mask(dataframe, index, mask)
    return dataframe


def get_difference_from(dataframe, other, index):
    mask = _get_index_mask(dataframe, other, index, 'diff')
    dataframe = _filter_by_index_mask(dataframe, index, mask)
    return dataframe



from collections import abc
import sqlite3

import pandas as pd
import pymysql
from sqlalchemy import create_engine

from scrapgo.utils.fileutils import get_file_extension, read_json, read_conf
from .dfutils import get_difference_from


QUERY_FORMAT = {
    'select': "SELECT {columns} FROM {table}",
    'select-where': "SELECT {columns} FROM {table} WHERE {where}",
    'max': "SELECT MAX({column}) FROM {table}",
    'max-where': "SELECT MAX({column}) FROM {table} WHERE {where}",
    'min': "SELECT MIN({column}) FROM {table}",
    'min-where': "SELECT MIN({column}) FROM {table} WHERE {where}",
    'count': "SELECT COUNT(*) FROM {table}",
    'count-where': "SELECT COUNT(*) FROM {table} WHERE {where}",
    'delete': "DELETE FROM {table}",
    'delete-where': "DELETE FROM {table} WHERE {where}",
}


def _join_columns(df, column, type=str):
    if type is str:
        clause = ','.join(f'"{k}"' for k in df[column].values)
    else:
        clause = ','.join(df[column].values)
    return clause  

class TableFrame(object):

    def __init__(self, db_conf_path=None, **conn_info):
        self.con = None
        if db_conf_path is not None:
            ext = get_file_extension(db_conf_path)
            if ext in ['.json']:
                conn_info = read_json(db_conf_path)
            elif ext in ['.cnf', '.conf']:
                conn_info = read_conf(db_conf_path, header='database')
            else:
                raise ValueError(
                    f"{ext} is not DB connection setting file extension(Only support .json, .cnf, .conf)"
                )  
        self.con = self._get_db_connection(**conn_info)

    def __del__(self):
        if self.con:
            self.con.close()

    def _get_db_connection(self, **kwargs):
        db_backend = kwargs.pop('backend', 'sqlite3')
        if db_backend == 'sqlite3': 
            con = sqlite3.connect(**kwargs)
        elif db_backend == 'mysql':
            con_str_fmt = "mysql+pymysql://{user}:{passwd}@{host}:{port}/{db}?charset={charset}"
            con_str = con_str_fmt.format(**kwargs)
            engin = create_engine(con_str, encoding='utf-8')
            con = engin.connect()
        else:
            raise ValueError("DB backend only supported in sqlite3, mysql")
        return con
    
    def close(self):
        self.con.close()

    def _get_now(self):
        return pd.Timestamp.now()

    def _get_query(self, kind, table, query_formats=QUERY_FORMAT, **kwargs):
        fmt = query_formats[kind]
        return fmt.format(table=table, **kwargs)

    def get_dataframe(self, query, **kwargs):
        try:
            dataframe = pd.read_sql(query, self.con, **kwargs)
        except Exception as e:
            print(e)
        else:
            return dataframe

    def select(self, table, columns=None, where=None):
        if columns is None:
            columns = '*'
        else:
            columns = ','.join(columns)
        if where is None:
            query = self._get_query('select', table, columns=columns)
        else:
            query = self._get_query(
                'select-where', table, columns=columns, where=where
            )
        df = self.get_dataframe(query)
        return df

    def max(self, table, column, where=None, **kwargs):
        if where is None:
            query = self._get_query('max', table, column=column, **kwargs)
        else:
            query = self._get_query(
                'max-where', table,
                column=column, where=where, **kwargs
            )
        df = self.get_dataframe(query)
        if df is not None:
            return df.loc[0, f"MAX({column})"]

    def min(self, table, column, where=None, **kwargs):
        if where is None:
            query = self._get_query('min', table, column=column, **kwargs)
        else:
            query = self._get_query(
                'min-where', table,
                column=column, where=where, **kwargs
            )
        df = self.get_dataframe(query)
        if df is not None:
            return df.loc[0, f"MIN({column})"]

    def count(self, table, where=None, **kwargs):
        if where is None:
            query = self._get_query('count', table, **kwargs)
        else:
            query = self._get_query(
                'count-where', table, where=where, **kwargs
            )
        df = self.get_dataframe(query)
        if df is not None:
            return df.loc[0, "COUNT(*)"]
        return 0

    def insert(self, dataframe, table, uniques, renames=None, updated=None, created=None, if_exists='append', logging=True, **kwargs):
        if dataframe.empty is True:
            if logging is True:
                print(f"Dataframe is empty {0} ROW(s) was Inserted into {table} ({self._get_now()})")
            return

        if renames is not None:
            dataframe = dataframe.rename(columns=renames)
        
        if if_exists == 'append':
            df_table = self.select(table, uniques, **kwargs)
            if df_table is not None:
                dataframe = get_difference_from(dataframe, df_table, uniques)

        if dataframe.empty is False:            
            if updated is not None:
                dataframe[updated] = self._get_now()
            if created is not None:
                dataframe[created] = self._get_now()

            dataframe.to_sql(
                table, self.con,
                if_exists=if_exists,
                index=False
            )
        if logging is True:
            count = dataframe.shape[0]
            print(f"{count} ROW(s) was Inserted into {table} ({self._get_now()})")

    
    def update(self, dataframe, table, source_column, dest_column, valuemap=None, on=None, index=None, pk=None, default=None, value_type=str, pk_type=str, logging=True):
        if dataframe.empty is True:
            if logging is True:
                print(f"Dataframe is empty {0} ROW(s) was Updated to {table} ({self._get_now()})")
            return

        fmt = "UPDATE {table} SET {column}={value} WHERE {column} != {value} AND {pk} IN ({pk_values})"
        for value, df in dataframe.groupby(source_column):
            if isinstance(valuemap, abc.Mapping):
                value = valuemap.get(value, default or value)
            elif callable(valuemap):
                value = valuemap(value)
            else:
                value = default or value
            
            if value_type is str:
                value = f'"{value}"'
            pk_values = _join_columns(df, on or index, pk_type)
            query = fmt.format(
                table=table,
                column=dest_column,
                value=value,
                pk= on or pk,
                pk_values=pk_values,    
            )
            if logging is True:
                count = df.shape[0]
                print(f"{count} ROW(s) was Updated into {table} by Query: {query[:80]}... ({self._get_now()})")
            self.con.execute(query)
    
    def delete(self, table, where=None, logging=True):
        if where is None:
            count = self.count(table)
            query = self._get_query('delete', table=table)
        else:
            count = self.count(table, where=where)
            query = self._get_query(
                'delete-where', table=table, where=where
            )
        self.con.execute(query)
        if logging is True:
            print(f"{count} ROW(s) was Deleted by Query: {query}")



